diff mbox

[U-Boot,v3,1/2] drivers/video/am335x-fb: Add support for 16bpp format

Message ID 1461785956-30309-1-git-send-email-martin.pietryka@chello.at
State Accepted
Commit 7d045170ac1bf512ed148e97432b7a8d772943e9
Delegated to: Anatolij Gustschin
Headers show

Commit Message

Martin Pietryka April 27, 2016, 7:39 p.m. UTC
To support 16bpp we just need to change the raster_ctrl register
accordingly. Also 32bpp mode should work as well, but was not tested.
According to the TRM the uppermost byte will be ignored when
LCD_TFT_24BPP_UNPACK is set.

The switch logic is based on the Liunx kernel tilcdc driver:
drivers/gpu/drm/tilcdc/tilcdc_crtc.c: lines 407 through 419
(kernel was checked out at commit: bcc981e9ed8)

Signed-off-by: Martin Pietryka <martin.pietryka@chello.at>
---

Changes in v3:
- Moved switch statement before changing any registers

 drivers/video/am335x-fb.c | 22 +++++++++++++++++++---
 1 file changed, 19 insertions(+), 3 deletions(-)

Comments

Hannes Schmelzer April 28, 2016, 4:59 a.m. UTC | #1
On 27.04.2016 21:39, Martin Pietryka wrote:
> To support 16bpp we just need to change the raster_ctrl register
> accordingly. Also 32bpp mode should work as well, but was not tested.
> According to the TRM the uppermost byte will be ignored when
> LCD_TFT_24BPP_UNPACK is set.
>
> The switch logic is based on the Liunx kernel tilcdc driver:
> drivers/gpu/drm/tilcdc/tilcdc_crtc.c: lines 407 through 419
> (kernel was checked out at commit: bcc981e9ed8)
>
> Signed-off-by: Martin Pietryka <martin.pietryka@chello.at>
> ---
>
> Changes in v3:
> - Moved switch statement before changing any registers
>
>   drivers/video/am335x-fb.c | 22 +++++++++++++++++++---
>   1 file changed, 19 insertions(+), 3 deletions(-)
>
> diff --git a/drivers/video/am335x-fb.c b/drivers/video/am335x-fb.c
> index 0ec843f..d984435 100644
> --- a/drivers/video/am335x-fb.c
> +++ b/drivers/video/am335x-fb.c
> @@ -5,7 +5,7 @@
>    * minimal framebuffer driver for TI's AM335x SoC to be compatible with
>    * Wolfgang Denk's LCD-Framework (CONFIG_LCD, common/lcd.c)
>    *
> - * - supporting only 24bit RGB/TFT raster Mode (not using palette)
> + * - supporting 16/24/32bit RGB/TFT raster Mode (not using palette)
>    * - sets up LCD controller as in 'am335x_lcdpanel' struct given
>    * - starts output DMA from gd->fb_base buffer
>    *
> @@ -106,6 +106,8 @@ int lcd_get_size(int *line_length)
>   
>   int am335xfb_init(struct am335x_lcdpanel *panel)
>   {
> +	u32 raster_ctrl = 0;
> +
>   	if (0 == gd->fb_base) {
>   		printf("ERROR: no valid fb_base stored in GLOBAL_DATA_PTR!\n");
>   		return -1;
> @@ -115,6 +117,21 @@ int am335xfb_init(struct am335x_lcdpanel *panel)
>   		return -1;
>   	}
>   
> +	/* We can already set the bits for the raster_ctrl in this check */
> +	switch (panel->bpp) {
> +	case 16:
> +		break;
> +	case 32:
> +		raster_ctrl |= LCD_TFT_24BPP_UNPACK;
> +		/* fallthrough */
> +	case 24:
> +		raster_ctrl |= LCD_TFT_24BPP_MODE;
> +		break;
> +	default:
> +		error("am335x-fb: invalid bpp value: %d\n", panel->bpp);
> +		return -1;
> +	}
> +
>   	debug("setting up LCD-Controller for %dx%dx%d (hfp=%d,hbp=%d,hsw=%d / ",
>   	      panel->hactive, panel->vactive, panel->bpp,
>   	      panel->hfp, panel->hbp, panel->hsw);
> @@ -157,8 +174,7 @@ int am335xfb_init(struct am335x_lcdpanel *panel)
>   				LCD_HBPMSB(panel->hbp) |
>   				LCD_HFPMSB(panel->hfp) |
>   				0x0000FF00;	/* clk cycles for ac-bias */
> -	lcdhw->raster_ctrl =	LCD_TFT_24BPP_MODE |
> -				LCD_TFT_24BPP_UNPACK |
> +	lcdhw->raster_ctrl =	raster_ctrl |
>   				LCD_PALMODE_RAWDATA |
>   				LCD_TFT_MODE |
>   				LCD_RASTER_ENABLE;
Reviewed-by: Hannes Schmelzer <oe5hpm@oevsv.at>
Tested-by: Hannes Schmelzer <oe5hpm@oevsv.at>
Anatolij Gustschin April 28, 2016, 2:56 p.m. UTC | #2
On Wed, 27 Apr 2016 21:39:15 +0200
Martin Pietryka martin.pietryka@chello.at wrote:

> To support 16bpp we just need to change the raster_ctrl register
> accordingly. Also 32bpp mode should work as well, but was not tested.
> According to the TRM the uppermost byte will be ignored when
> LCD_TFT_24BPP_UNPACK is set.
> 
> The switch logic is based on the Liunx kernel tilcdc driver:
> drivers/gpu/drm/tilcdc/tilcdc_crtc.c: lines 407 through 419
> (kernel was checked out at commit: bcc981e9ed8)
> 
> Signed-off-by: Martin Pietryka <martin.pietryka@chello.at>
> ---
> 
> Changes in v3:
> - Moved switch statement before changing any registers
> 
>  drivers/video/am335x-fb.c | 22 +++++++++++++++++++---
>  1 file changed, 19 insertions(+), 3 deletions(-)

appied to u-boot-video/master, thanks!

--
Anatolij
diff mbox

Patch

diff --git a/drivers/video/am335x-fb.c b/drivers/video/am335x-fb.c
index 0ec843f..d984435 100644
--- a/drivers/video/am335x-fb.c
+++ b/drivers/video/am335x-fb.c
@@ -5,7 +5,7 @@ 
  * minimal framebuffer driver for TI's AM335x SoC to be compatible with
  * Wolfgang Denk's LCD-Framework (CONFIG_LCD, common/lcd.c)
  *
  * - supporting only 24bit RGB/TFT raster Mode (not using palette)
  * - sets up LCD controller as in 'am335x_lcdpanel' struct given
  * - starts output DMA from gd->fb_base buffer
  *
@@ -106,6 +106,8 @@  int lcd_get_size(int *line_length)
 
 int am335xfb_init(struct am335x_lcdpanel *panel)
 {
+	u32 raster_ctrl = 0;
+
 	if (0 == gd->fb_base) {
 		printf("ERROR: no valid fb_base stored in GLOBAL_DATA_PTR!\n");
 		return -1;
@@ -115,6 +117,21 @@  int am335xfb_init(struct am335x_lcdpanel *panel)
 		return -1;
 	}
 
+	/* We can already set the bits for the raster_ctrl in this check */
+	switch (panel->bpp) {
+	case 16:
+		break;
+	case 32:
+		raster_ctrl |= LCD_TFT_24BPP_UNPACK;
+		/* fallthrough */
+	case 24:
+		raster_ctrl |= LCD_TFT_24BPP_MODE;
+		break;
+	default:
+		error("am335x-fb: invalid bpp value: %d\n", panel->bpp);
+		return -1;
+	}
+
 	debug("setting up LCD-Controller for %dx%dx%d (hfp=%d,hbp=%d,hsw=%d / ",
 	      panel->hactive, panel->vactive, panel->bpp,
 	      panel->hfp, panel->hbp, panel->hsw);
@@ -157,8 +174,7 @@  int am335xfb_init(struct am335x_lcdpanel *panel)
 				LCD_HBPMSB(panel->hbp) |
 				LCD_HFPMSB(panel->hfp) |
 				0x0000FF00;	/* clk cycles for ac-bias */
-	lcdhw->raster_ctrl =	LCD_TFT_24BPP_MODE |
-				LCD_TFT_24BPP_UNPACK |
+	lcdhw->raster_ctrl =	raster_ctrl |
 				LCD_PALMODE_RAWDATA |
 				LCD_TFT_MODE |
 				LCD_RASTER_ENABLE;
@@ -5,7 +5,7 @@ 
  * Wolfgang Denk's LCD-Framework (CONFIG_LCD, common/lcd.c)
  *
- * - supporting only 24bit RGB/TFT raster Mode (not using palette)
+ * - supporting 16/24/32bit RGB/TFT raster Mode (not using palette)
  * - sets up LCD controller as in 'am335x_lcdpanel' struct given
  * - starts output DMA from gd->fb_base buffer